struct light
{
  vec3 position;
  vec4 ambient;
  vec4 diffuse;
  vec4 specular;
  vec3 halfVector;

  // only directional light is supported
};

uniform light u_lights[1];

uniform sampler2D   s_texture0; // Base
uniform sampler2D   s_texture1; // Light color
uniform samplerCube s_texture2; // Light cube

varying vec2 v_texCoord0;
varying vec3 v_worldC;
varying vec3 v_normal;
varying vec3 v_R;

void main()
{
  vec4 baseColor   = texture2D(s_texture0, v_texCoord0);
  vec4 lightColor  = texture2D(s_texture1, v_worldC.xz);
  vec4 sunMapColor = textureCube(s_texture2, v_R);
  
  // Lambert
  vec3 n,lightDir;
  float NdotL;
  
  lightDir = vec3(u_lights[0].position);
  
  /* The ambient term will always be present */
  vec4 color = lightColor;
  /* a fragment shader can't write a varying variable, hence we need
   a new variable to store the normalized interpolated normal */
  n = normalize(v_normal);
  /* compute the dot product between normal and ldir */
  
  NdotL = max(dot(n,lightDir),0.0);
  color += u_lights[0].diffuse * NdotL;
  color += sunMapColor; // Spec
  
  gl_FragColor = (baseColor * color);
  gl_FragColor.a = 1.0;
}