uniform sampler2D s_texture0;
uniform sampler2D s_texture1;

varying vec2 v_texCoord0;

uniform mat4 u_my_matVPInv;
uniform mat4 u_my_matVPPrev;

void main()
{
  vec4 depthValue = texture2D(s_texture1, v_texCoord0);
  float zOverW = depthValue.z;
  
  // H is the viewport position at this pixel in the range -1 to 1.
  vec4 H = vec4(v_texCoord0.x * 2.0 - 1.0,
                v_texCoord0.y * 2.0 - 1.0,
                zOverW,
                //2.0*zOverW - 1.0,
                1.0);
  
  // Transform by the view-projection inverse.
  vec4 D = u_my_matVPInv * H;
  
  // Divide by w to get the world position.
  vec4 worldPos = D / D.w;
  
  // Current viewport position
  vec4 currentPos = H;
  
  // Use the world position, and transform by the previous view-
  // projection matrix.
  vec4 previousPos = u_my_matVPPrev * worldPos;
  // Convert to nonhomogeneous points [-1,1] by dividing by w.
  previousPos /= previousPos.w;
  
  // Use this frame's position and last frame's to compute the pixel
  // velocity.
  vec2 velocity = (currentPos.xy - previousPos.xy) * 0.06;

  vec2 texCoord = v_texCoord0;
  
#if 1
  vec4 color = texture2D(s_texture0, texCoord);

  texCoord = v_texCoord0 - velocity * 4.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 - velocity * 3.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 - velocity * 2.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 - velocity * 1.0;
  color += texture2D(s_texture0, texCoord);
  

  texCoord = v_texCoord0 + velocity * 1.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 + velocity * 2.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 + velocity * 3.0;
  color += texture2D(s_texture0, texCoord);
  
  texCoord = v_texCoord0 + velocity * 4.0;
  color += texture2D(s_texture0, texCoord);
  
  vec4 finalColor = color * 1.0/9.0;
  
#else
  float numSamples = 6.0;
  float numSamples_rec 1.0/6.0;
  
  // Get the initial color at this pixel.
  vec4 color = texture2D(s_texture0, texCoord);
  
  texCoord += velocity;
  for(float i = 1.0; i < numSamples; i += 1.0, texCoord += velocity)
  {
    // Sample the color buffer along the velocity vector.
    vec4 currentColor = texture2D(s_texture0, texCoord);
    // Add the current color to our color sum.
    color += currentColor;
  }
  
  // Average all of the samples to get the final blur color.
  vec4 finalColor = color * numSamples_rec;
#endif
  
  finalColor.a = 1.0;
  
  gl_FragColor = finalColor;
}